<?php

namespace Drupal\gcommerce;

use Drupal\commerce_order\AddressBook as CoreAddressBook;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\GroupMembershipLoaderInterface;
use Drupal\profile\Entity\ProfileInterface;
use Drupal\user\UserInterface;

class AddressBook extends CoreAddressBook {

  /**
   * The membership loader service.
   *
   * @var \Drupal\group\GroupMembershipLoaderInterface
   */
  protected $membershipLoader;

  /**
   * Constructs a new AddressBook object.
   *
   * @param \Drupal\Core\Entity\EntityTypeBundleInfo $entity_type_bundle_info
   *   The entity type bundle info.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\group\GroupMembershipLoaderInterface $membership_loader
   *   The group membership loader service.
   */
  public function __construct(EntityTypeBundleInfo $entity_type_bundle_info, EntityTypeManagerInterface $entity_type_manager, GroupMembershipLoaderInterface $membership_loader) {
    parent::__construct($entity_type_bundle_info, $entity_type_manager);

    $this->membershipLoader = $membership_loader;
  }

  /**
   * {@inheritdoc}
   */
  public function loadAll(UserInterface $customer, $profile_type_id, array $available_countries = []) {
    $profiles = [];
    $list_only_group_profiles = FALSE;
    // Load the profiles belonging to groups the given customer is member of.
    foreach ($this->membershipLoader->loadByUser($customer) as $group_membership) {
      try {
        $group = $group_membership->getGroup();
        // Check whether only profiles belonging to groups should be returned
        // by the Addressbook.
        $configuration = $group->getGroupType()->getContentPlugin("group_profile:$profile_type_id")->getConfiguration();
        // @todo: Check what to do when profiles from different group types
        // with different configuration is returned.
        if (!empty($configuration['addressbook_list_group_profiles'])) {
          $list_only_group_profiles = TRUE;
        }
        foreach ($group->getContentEntities("group_profile:$profile_type_id") as $profile) {
          if (!$this->isAvailable($profile, $available_countries)) {
            continue;
          }
          $profiles[$profile->id()] = $profile;
        }
      }
      catch (PluginNotFoundException $exception) {
        continue;
      }
    }

    // Skip returning profiles belonging to the current customer if configured
    // to do so.
    if (!$list_only_group_profiles) {
      $profiles += parent::loadAll(
        $customer,
        $profile_type_id,
        $available_countries
      );
    }

    return $profiles;
  }

  /**
   * {@inheritdoc}
   */
  public function copy(ProfileInterface $profile, UserInterface $customer) {
    parent::copy($profile, $customer);
    // Add the addressbook profile to customer's groups.
    if ($profile->getData('address_book_profile_id')) {
      $addressbook_profile = $this->profileStorage->load($profile->getData('address_book_profile_id'));

      // @todo Add functionality that will load allowed groups.
      // Allowed groups for which can be added card entity.
      $allowedGroupTypeIds = ['company'];

      $groups = $this->membershipLoader->loadByUser($customer);
      foreach ($groups as $membership) {
        $group = $membership->getGroup();
        $groupType = $group->getGroupType();
        if (!in_array($groupType->id(), $allowedGroupTypeIds)) {
          continue;
        }
        $contentPluginId = 'group_profile:' . $addressbook_profile->bundle();
        if ($groupType->hasContentPlugin($contentPluginId)
          && empty($group->getContentByEntityId($contentPluginId, $addressbook_profile->id()))) {
          $group->addContent($profile, $contentPluginId);
        }
      }
    }
  }

}
